@@ -12,7 +12,7 @@ import logging
 
 from bluetooth_data_tools import short_address
 from bluetooth_sensor_state_data import BluetoothData
-from home_assistant_bluetooth import BluetoothServiceInfo
+from habluetooth import BluetoothServiceInfoBleak
 from sensor_state_data import SensorLibrary
 from sensor_state_data.description import BaseSensorDescription
 
@@ -145,7 +145,7 @@ def decode_values(
 
 
 def determine_device_type(
-    service_info: BluetoothServiceInfo, manufacturer_data: dict[int, bytes]
+    service_info: BluetoothServiceInfoBleak, manufacturer_data: dict[int, bytes]
 ) -> str | None:
     """Determine the device type based on the name and UUID"""
     local_name = service_info.name
@@ -168,7 +168,7 @@ def determine_device_type(
 class SensorPushBluetoothDeviceData(BluetoothData):
     """Date update for SensorPush Bluetooth devices."""
 
-    def _start_update(self, service_info: BluetoothServiceInfo) -> None:
+    def _start_update(self, service_info: BluetoothServiceInfoBleak) -> None:
         """Update from BLE advertisement data."""
         manufacturer_data = service_info.manufacturer_data
         if not manufacturer_data:
